package com.atomikos.examples.hibernate;

import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.transaction.UserTransaction;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

import com.atomikos.examples.hibernate.entities.User;
import com.atomikos.jdbc.SimpleDataSourceBean;

public class Main {
	
	private static UserTransaction userTransaction;
	private static SimpleDataSourceBean dataSource1;
	private static SimpleDataSourceBean dataSource2;

	public static void main(String[] args) throws Exception {
		initDataSources();
		initTransactionManager();
		SessionFactory sf1 = new Configuration().configure("/hibernate1.cfg.xml").buildSessionFactory();
		SessionFactory sf2 = new Configuration().configure("/hibernate2.cfg.xml").buildSessionFactory();
		
        userTransaction.setTransactionTimeout(60);
		userTransaction.begin();
		
		try {
			System.out.println("*** DB1 ***");
			doWork(sf1, "user");

			System.out.println("*** DB2 ***");
			doWork(sf2, "user");

			userTransaction.commit();
		}
		catch (Exception ex) {
			ex.printStackTrace();
			userTransaction.rollback();
		}
		
		sf1.close();
		sf2.close();
		dataSource1.close();
		dataSource2.close();
		
		System.out.println("Both databases updated successfully");
	}

	/**
	 * Perform some work. In this scenario, we get a session from the specified
	 * SessionFactory then we create a new user with the specified username
	 * then list all users.  
	 */
	private static void doWork(SessionFactory sf, String username) {
		Session session = sf.getCurrentSession();
		persistUser(session, username);
		listUsers(session);
		session.close();
	}

	/**
	 * Dump all users to stdout using the specified Hibernate session. 
	 */
	private static void listUsers(Session session) {
		List users = session.createQuery("from User").list();
		for (int i=0; i<users.size() ;i++) {
			User user = (User) users.get(i);
			
			System.out.println(user.toString());
		}
	}

	/**
	 * Create a new user with specified name using the specified Hibernate session.
	 */
	private static void persistUser(Session session, String userName) {
		User u = new User();
		u.setName(userName);
		session.save(u);
	}

	/**
	 * Initialize the transaction manager.
	 */
	private static void initTransactionManager() {
		userTransaction = new com.atomikos.icatch.jta.UserTransactionImp();
	}

	/**
	 * Create the 2 datasources and bind them into the embedded JNDI server.
	 */
	private static void initDataSources() throws NamingException {
		Context ctx = new InitialContext();
		dataSource1 = getDataSource_db1();
		ctx.rebind("testDS1", dataSource1);
		dataSource2 = getDataSource_db2();
		ctx.rebind("testDS2", dataSource2);
		ctx.close();
	}

	/**
	 * Create the first datasource.
	 */
	private static SimpleDataSourceBean getDataSource_db1() {
		SimpleDataSourceBean ds = new SimpleDataSourceBean();
		ds.setUniqueResourceName("derby1");
		ds.setXaDataSourceClassName("org.apache.derby.jdbc.EmbeddedXADataSource");
		ds.setXaDataSourceProperties("databaseName=users1");
		ds.setConnectionPoolSize(3);
		return ds;
	}
	
	/**
	 * Create the second datasource.
	 */
	private static SimpleDataSourceBean getDataSource_db2() {
		SimpleDataSourceBean ds = new SimpleDataSourceBean();
		ds.setUniqueResourceName("derby2");
		ds.setXaDataSourceClassName("org.apache.derby.jdbc.EmbeddedXADataSource");
		ds.setXaDataSourceProperties("databaseName=users2");
		ds.setConnectionPoolSize(3);
		return ds;
	}

}
